﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Collections;
using System.Drawing;

namespace EditorRobobo
{
    class AStar
    {
        private Mapa mundo;
        private Point tam;

        public AStar(Mapa mundo)
        {
            this.mundo = mundo;
            this.tam = mundo.Size;
        }

        public List<NodoAStar> AEstrella()
        {
            List <NodoAStar> solucion = new List<NodoAStar>();
            int result = -1;
            HashSet<NodoAStar> listaInterior = new HashSet<NodoAStar>();
            PriorityQueue<NodoAStar, float> listaFrontera = new PriorityQueue<NodoAStar, float>();
            NodoAStar nodoinicial = new NodoAStar(null, NodoAStar.Origen, 0);
            float tmp=nodoinicial.H;
            listaFrontera.Enqueue(nodoinicial, 0);

            while (listaFrontera.Count > 0 && result!=0)
            {
                PriorityQueueItem<NodoAStar, float> item = listaFrontera.Dequeue();
                NodoAStar n = item.Value;
                listaInterior.Add(n);
                if (n.EsMeta())
                {
                    result = 0;
                    solucion = ReconstruirCamino(n);   
                }
                else
                {
                    List<NodoAStar> nodos = ExpandirNodos(n, listaInterior);
                    for (int i = 0; i < nodos.Count; i++)
                    {
                        NodoAStar m = nodos[i];
                        PriorityQueueItem<NodoAStar, float> mitem = new PriorityQueueItem<NodoAStar, float>(m, m.H*-1);
                        if (!listaFrontera.Contains(mitem))
                        {
                            tmp = m.H*-1; 
                            listaFrontera.Enqueue(m, tmp);
                        }
                        else
                        {
                            IEnumerator en=listaFrontera.GetEnumerator();
                            bool encontrado = false;
                            while (listaFrontera.Count > 0 && !encontrado)
                            {
                                en.MoveNext();
                                PriorityQueueItem<NodoAStar, float> auxitem = (PriorityQueueItem<NodoAStar, float>)en.Current;
                                NodoAStar aux = auxitem.Value;
                                if (aux.Equals(m))
                                {
                                    n = aux;
                                    encontrado = true;
                                    // TODO m.g=n.g, es necesario?
                                    if (m.G < n.G)
                                    {
                                        listaFrontera.Remove(aux);
                                        tmp=m.H;
                                        listaFrontera.Enqueue(mitem);
                                    }
                                }
                               
                            }
                        }
                    }
                }

            }
            return solucion;
        }

        private List<NodoAStar> ReconstruirCamino(NodoAStar n)
        {
            // Reconstruyo la solución, paso por paso
            List<NodoAStar> sol = new List<NodoAStar>();
            NodoAStar nodo = n;
            while (nodo != null)
            {
                sol.Add(nodo);
                nodo = nodo.Padre;
            }

            // Corro 1 posicion las direcciones
            for(int i=sol.Count-1;i>=0;i--)
            {
                nodo = sol[i];
                if (i >0)
                {
                    nodo.Direccion = sol[i -1].Direccion;
                }
                else
                    nodo.Direccion=Direcciones.SinAsignar;
            }
            return sol;
        }

        private List<NodoAStar> ExpandirNodos(NodoAStar n, HashSet<NodoAStar> listaInterior)
        {
            List<NodoAStar> nodos = new List<NodoAStar>();
            int[,] posibles=new int[4,2]{{1,0},{-1,0},{0,-1},{0,1}};
            Direcciones[] direcciones = new Direcciones[4] { Direcciones.Derecha, Direcciones.Izquierda, Direcciones.Arriba, Direcciones.Abajo };

            for (int i = 0; i < 4; i++)
            {
                int x = posibles[i, 0];
                int y = posibles[i, 1];

                if (ComprobarValidezPosicion(n, x, y))
                {
                    NodoAStar aux = new NodoAStar(n, new Point(n.Posicion.X + x, n.Posicion.Y + y), 1);
                    aux.Direccion = direcciones[i];
                    if(!listaInterior.Contains(aux))
                        nodos.Add(aux);
                }
            }
            return nodos;
        }

        private bool ComprobarValidezPosicion(NodoAStar n, int i, int j)
        {
            bool ret = false;

            // Si no es un borde
            if (n.Posicion.X + i >= 0 && n.Posicion.X + i < tam.X && n.Posicion.Y + j >= 0 && n.Posicion.Y + j < tam.Y)
            {
                // Si no es un obstaculo, se crea el nodo (y,x)=filasXcols?
                //if (mundo[n.Posicion.X + i,n.Posicion.Y + j] == 0)
                if(mundo.getCasilla(n.Posicion.X+i,n.Posicion.Y+j).Tipo!=TipoCasilla.Pared)
                {
                    ret = true;
                }

            }

            return ret;
        }
    }
}
