﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Diagnostics;

namespace EditorRobobo
{
    public partial class GUIMapa : UserControl
    {
        Rectangle _thisRect;
        Rectangle _thisRectOnPaint = new Rectangle();
        Rectangle _borderRectOnPaint = new Rectangle();
        Mapa map=new Mapa(40,22);
        Point _pos = new Point(-1, -1);
        bool _mouseDown=false;

       
        public GUIMapa()
        {
            InitializeComponent();
            _thisRect = this.Bounds;
            _thisRectOnPaint.Height = _thisRectOnPaint.Width = 24;
            _borderRectOnPaint.Height = _borderRectOnPaint.Width = 23;
        }

        public Mapa Map
        {
            get{return this.map;}
            set{
                this.map=value;
                this.Invalidate();
            }
        }

        public Point Pos
        {
            get { return _pos; }
        }

        protected override void OnMouseDown(MouseEventArgs e)
        {
            base.OnMouseDown(e);
            _mouseDown = true;
            this.Invalidate();
        }

        protected override void OnMouseUp(MouseEventArgs e)
        {
            base.OnMouseUp(e);
            _mouseDown = false; 
           
                int posX = e.X / 24;
                int posY = e.Y / 24;
                Casilla c = map.getCasilla(posX, posY);
                if (c != null)
                {
                    if (e.Button == MouseButtons.Left)
                    {
                        if (c.Tipo != TipoCasilla.Vacia)
                            c.Tipo = TipoCasilla.Vacia;
                        else if (c.Tipo != TipoCasilla.Pared)
                            c.Tipo = TipoCasilla.Pared;
                    }
                    else if (e.Button == MouseButtons.Right)
                    {
                        Casilla oldc = map.getCasilla(NodoAStar.Origen.X, NodoAStar.Origen.Y);
                        oldc.Tipo = TipoCasilla.Vacia;
                        c.Tipo = TipoCasilla.Ladron;
                        NodoAStar.Origen = new Point(posX, posY); // TODO el origen cambiará
                    }
                    else if (e.Button == MouseButtons.Middle)
                    {
                        Casilla oldc = map.getCasilla(NodoAStar.Destino.X, NodoAStar.Destino.Y);
                        oldc.Tipo = TipoCasilla.Vacia;
                        c.Tipo = TipoCasilla.Poli; // TODO el destino será o salida o joya
                        NodoAStar.Destino = new Point(posX, posY);
                    }
                    this.Invalidate();
                }
          
        }

        protected override void OnMouseLeave(EventArgs e)
        {
            base.OnMouseLeave(e);
            _pos.X = _pos.Y = -1;
            this.Invalidate();

            
        }
       

        Color Casilla2Color(Casilla c)
        {
            Color ret=Color.White;

            switch (c.Tipo)
            {
                case TipoCasilla.Pared:
                    ret=Color.Blue;
                    break;
                case TipoCasilla.Ladron:
                    ret = Color.Green;
                    break;
                case TipoCasilla.Poli:
                    ret = Color.Red;
                    break;
                case TipoCasilla.Joya:
                    ret = Color.Yellow;
                    break;
            }
            return ret;
        }

        protected override void OnPaint(PaintEventArgs e)
        {
            base.OnPaint(e);
            Graphics graphicsContext = e.Graphics;
            string str = "";

            for (int i = 0, startX = 0; i < 40; i++, startX += 24)
            {
                for (int j = 0, startY = 0; j < 22; j++, startY += 24)
                {
                    _thisRectOnPaint.X = startX;
                    _thisRectOnPaint.Y = startY;

                    Color colorHere=Color.White;
                    Casilla c = map.getCasilla(i, j);
                    if (c != null)
                    {
                        colorHere = Casilla2Color(map.getCasilla(i, j));
                        Direcciones dir = c.Direccion;
                        str = " ";
                        if (dir != Direcciones.SinAsignar)
                            str = dir.ToString().Substring(0,2);
                    }
                    // Pinto el origen y el destino con rectangulos vacios
                    Rectangle orig = new Rectangle(NodoAStar.Origen.X * 24, NodoAStar.Origen.Y * 24 , 24, 24);
                    Rectangle dst = new Rectangle(NodoAStar.Destino.X * 24, NodoAStar.Destino.Y * 24, 24, 24);
                    graphicsContext.DrawRectangle(Pens.OliveDrab, orig);
                    graphicsContext.DrawRectangle(Pens.IndianRed, dst);


                    if (_pos.X == i && _pos.Y == j)
                    {
                        if (_mouseDown)
                        {
                            graphicsContext.FillRectangle(Brushes.Black, _thisRectOnPaint);
                        }
                        else
                        {
                            graphicsContext.FillRectangle(new SolidBrush(colorHere), _thisRectOnPaint);

                            _borderRectOnPaint.X = startX;
                            _borderRectOnPaint.Y = startY;

                            graphicsContext.DrawRectangle(Pens.Black, _borderRectOnPaint);
                            
                        }
                    }
                    else
                    {
                        if (colorHere != Color.White)
                        {
                            Brush brush = new SolidBrush(colorHere);
                            graphicsContext.FillRectangle(brush, _thisRectOnPaint);
                            
                        }
                        else
                            graphicsContext.DrawString(str.ToString(), this.Font, Brushes.Black, _thisRectOnPaint);
                    }
                }
            }
        }

        private void GUIMapa_Load(object sender, EventArgs e)
        {

        }
    }
    
}
